// Copyright  1992-1993 Emergent Behavior. All rights reserved.

//=================================================================
#ifndef _TLANDERDOCUMENT_
	#include "TLanderDocument.h"
#endif

#ifndef _TLANDERVIEW_
	#include "TLanderView.h"
#endif

//--- from Physics framnework ----------------------------------------
#ifndef _PHYSICS_
	#include <Physics.h>
#endif

//=================================================================
const long  kCommandThreeResult = (kCommandsMenuID<<16) + 3;

//=================================================================
TLanderDocument::TLanderDocument( Scalar initialAlt )
	: TDocument(), MIdler(0), fThrust (0,0), fInitialTicks( TickCount() )
{
	this->SetMissionInProgress (true );
	this->SetInitialAltitude ( initialAlt );
	Scalar kMoonRadius	= 1740000;
	fSystem = NIL;
	fSystem	= new TSystem();
	fSystem->AddForceType( new TGravForce() );
	
	//---------------ship-------------------
	// must add ship first !!!!!!!
	const StandardPrecision kThrustEfficiencyRatio = 2500.0;
	const Mass kEngineMass = 1000.0;
	const Distance kEngineRadius = 2.0;
	TEngine*	engine = new TEngine(kThrustEfficiencyRatio, kEngineMass, kEngineRadius);
	
	const Distance kRadiusInMeters = 1.0;
	const Mass kTankMass = 1000.0;
	const Mass kFuelMass = 10000.0;
	TFuelTank*	tank = new TFuelTank(kRadiusInMeters, kTankMass, kFuelMass);

	TSESpaceship* ship	= new TSESpaceship ( TRealPoint ( 350, 100 ), 
											TVector( 0,0 ), 10, 50000, 
											fSystem, engine, tank );
	fSystem->AddBody( ship );

	//---------------moon------------------
	TBody* moon = new TSolarBody(	TSolarBody::kMoon,
									TRealPoint (350, kMoonRadius +initialAlt ), 
									TVector(0, 0) ); 
	fSystem->AddBody( moon );
}

//-----------------------------------------------------------------
TLanderDocument::TLanderDocument(const TLanderDocument& source)
	: TDocument(source), 
	MIdler( source ), 
	fInitialAltitude ( source.fInitialAltitude ),
	fThrust ( source.fThrust ),
	fMissionInProgress (source.fMissionInProgress ),
	fInitialTicks( TickCount() )
{
	fSystem = (TSystem*)( source.fSystem->Clone() );
}

//-----------------------------------------------------------------
MQACollectible*
TLanderDocument::Clone() const
{
	return (TDocument*)new TLanderDocument(*this);
}

//-----------------------------------------------------------------
TLanderDocument::~TLanderDocument()
{
}
//-----------------------------------------------------------------
void		
TLanderDocument::SetThrust( TThrustForce& newThrust )
{
	fThrust = newThrust;
}

//-----------------------------------------------------------------
void
TLanderDocument::SetInitialAltitude(Scalar meters)
{
	fInitialAltitude = meters;
}

//-----------------------------------------------------------------
void
TLanderDocument::SetFinalSpeed(Scalar metersPerSecond)
{
	fFinalSpeed = metersPerSecond;
}

//-----------------------------------------------------------------
long
TLanderDocument::GetInitialTicks()
{
	return fInitialTicks;
}

//-----------------------------------------------------------------
void
TLanderDocument::SetMissionInProgress(Boolean inProgress)
{
	fMissionInProgress = inProgress;
}

//-----------------------------------------------------------------
void
TLanderDocument::DoSetupMenus(TMenuBar& theMenuBar)
{
	Boolean idling = this->IsActive();		// from MIdler
	if ( ! idling )
		theMenuBar.Enable(kCommandOneResult);
	else
		theMenuBar.Enable(kCommandTwoResult);
	TDocument::DoSetupMenus(theMenuBar);
}

//-----------------------------------------------------------------
void
TLanderDocument::DoMenuCommand(long menuResult)
{
	switch (menuResult) {

	case kCommandOneResult:
		this->StartIdler();
		fInitialTicks = TickCount();
		break;
		
	case kCommandTwoResult:
		this->StopIdler();
		break;
		
	default:
		TDocument::DoMenuCommand(menuResult);
		}	// switch
}

//-----------------------------------------------------------------
TView*
TLanderDocument::DoCreateViews()
{
	const Point kViewSize = {293, 401};		// height, width
	return new TLanderView(kViewSize, this, fInitialAltitude);
}

//-----------------------------------------------------------------
TSystem*
TLanderDocument::GetSystem()
{
	return fSystem;
}


//-----------------------------------------------------------------
void
TLanderDocument::DoIdle()
{
	if ( fMissionInProgress ) {
		TListIterator iter = this->GetSystem()->GetBodyIterator();
		TSESpaceship* ship =  (TSESpaceship*)iter.First();
		long startTicks = TickCount();
		long currentTicks = startTicks;
		float kTimeStepSize	= .25;				// sec
		if ( fThrust.GetLength() != 0 )
			ship->Thrust ( fThrust, kTimeStepSize );
		fSystem->NextTimeStep(kTimeStepSize);
		fWindow->InvalidateView();
		fWindow->DrawView();
		while ( (TickCount() - startTicks ) < 15 ) {}
	}
	else {
		this->StopIdler();
		if ( fFinalSpeed >= 4 )
			TApplication::StdAlert ( 198 );
		else
			TApplication::StdAlert ( 199 );
	}
}
